module GreenhouseIo
  class Client
    include GreenhouseIo::API

    PERMITTED_OPTIONS = [:page, :per_page, :job_id]
    PERMITTED_OPTIONS_PER_ENDPOINT = {
      'offers' => [:status, :resolved_after],
      'candidates' => [:updated_after, :email]
    }

    attr_accessor :api_token, :rate_limit, :rate_limit_remaining, :link
    BASE_URL = 'https://harvest.greenhouse.io/v1'

    def initialize(api_token = nil)
      @api_token = api_token || GreenhouseIo.configuration.api_token
    end

    def offices(id = nil, options = {})
      get_from_harvest_api "#{BASE_URL}/offices#{path_id(id)}", options
    end

    def offers(id = nil, options = {})
      get_from_harvest_api "#{BASE_URL}/offers#{path_id(id)}", options, "offers"
    end

    def departments(id = nil, options = {})
      get_from_harvest_api "#{BASE_URL}/departments#{path_id(id)}", options
    end

    def candidates(id = nil, options = {})
      get_from_harvest_api "#{BASE_URL}/candidates#{path_id(id)}", options, "candidates"
    end

    def activity_feed(id, options = {})
      get_from_harvest_api "#{BASE_URL}/candidates/#{id}/activity_feed", options
    end

    def create_candidate_note(candidate_id, note_hash, on_behalf_of)
      post_to_harvest_api(
        "#{BASE_URL}/candidates/#{candidate_id}/activity_feed/notes",
        note_hash,
        { 'On-Behalf-Of' => on_behalf_of.to_s }
      )
    end

    def applications(id = nil, options = {})
      get_from_harvest_api "#{BASE_URL}/applications#{path_id(id)}", options
    end

    def offers_for_application(id, options = {})
      get_from_harvest_api "#{BASE_URL}/applications/#{id}/offers", options
    end

    def current_offer_for_application(id, options = {})
      get_from_harvest_api "#{BASE_URL}/applications/#{id}/offers/current_offer", options
    end

    def scorecards(id, options = {})
      get_from_harvest_api "#{BASE_URL}/applications/#{id}/scorecards", options
    end

    def all_scorecards(id = nil, options = {})
      get_from_harvest_api "#{BASE_URL}/scorecards/#{id}", options
    end

    def scheduled_interviews(id, options = {})
      get_from_harvest_api "#{BASE_URL}/applications/#{id}/scheduled_interviews", options
    end

    def jobs(id = nil, options = {})
      get_from_harvest_api "#{BASE_URL}/jobs#{path_id(id)}", options
    end

    def stages(id, options = {})
      get_from_harvest_api "#{BASE_URL}/jobs/#{id}/stages", options
    end

    def job_post(id, options = {})
      get_from_harvest_api "#{BASE_URL}/jobs/#{id}/job_post", options
    end

    def job_posts(options = {})
      get_from_harvest_api "#{BASE_URL}/job_posts", options
    end

    def users(id = nil, options = {})
      get_from_harvest_api "#{BASE_URL}/users#{path_id(id)}", options
    end

    def sources(id = nil, options = {})
      get_from_harvest_api "#{BASE_URL}/sources#{path_id(id)}", options
    end

    private

    def path_id(id = nil)
      "/#{id}" unless id.nil?
    end

    def permitted_options(options)
      options.select { |key, value| PERMITTED_OPTIONS.include? key }
    end

    def permitted_options_for_endpoint(options, endpoint)
      options.select { |key, value| PERMITTED_OPTIONS_PER_ENDPOINT[endpoint].include? key }
    end

    def get_from_harvest_api(url, options = {}, endpoint = nil)
      all_permitted_options = permitted_options(options)
      all_permitted_options.merge!(permitted_options_for_endpoint(options, endpoint)) if endpoint

      response = get_response(url, all_permitted_options)

      if response.code == 200
        parse_json(response)
      else
        raise GreenhouseIo::Error.new(response.code)
      end
    end

    def post_to_harvest_api(url, body, headers)
      response = post_response(url, {:body => JSON.dump(body)}, headers)

      if response.code == 200
        parse_json(response)
      else
        raise GreenhouseIo::Error.new(response.code)
      end
    end
  end
end
